//
//  CAShapeLayer+Other.h
//  Test
//
//  Created by temp on 16/8/11.
//  Copyright © 2016年 ehoo100. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import <UIKit/UIKit.h>
#import "LGLChartUtil.h"


/* 结构体：虚线属性DottedLineAttributes */
struct DTLAttr {
    CGColorRef strokeColor;
    CGColorRef fillColor;
    CGFloat lineWidth;
    CGFloat dottedLength;
    CGFloat dottedSpace;
};
typedef struct DTLAttr DTLAttr;

// 内连函数：创建DTLAttr
static inline DTLAttr DTLAttrMake(CGColorRef strokeColor,
                                  CGColorRef fillColor,
                                  CGFloat lineWidth,
                                  CGFloat dottedLength,
                                  CGFloat dottedSpace) {
    DTLAttr attr = {strokeColor, fillColor, lineWidth, dottedLength, dottedSpace};
    return attr;
}


@interface CAShapeLayer (Other)

/**
 *  画线，加载到传入的view上
 *
 *  @param points 连接线的点
 *  @param view   线所在的view
 *  @param color  线条颜色
 *  @param width  线条宽度
 */

+ (instancetype)drawLine:(NSArray *)points onLayer:(CALayer *)layer strokeColor:(CGColorRef)strokeColor lineWidth:(CGFloat)lineWidth;


/**
 *  画一个图形，加载到传入的view上
 *
 *  @param points 图形边线上的点
 *  @param view   图形所在的view
 *  @param color  图形填充颜色
 *  @param color  图形边线颜色
 */

+ (instancetype)drawShape:(NSArray *)points onLayer:(CALayer *)layer fillColor:(CGColorRef)fillColor lineWidth:(CGFloat)lineWidth;

/**
 *  画虚线
 *
 *  @param points       虚线上的关键点
 *  @param pathClose    path是否闭合
 *  @param layer        虚线添加到的图层
 *  @param patterns     包含四个元素：strokeColor（UIColor *), fillColor(UIColor *), lineWidth(CGFloat), lineLengthAndSpace(NSArray *:@[dottedLength, space])
 */
+ (instancetype)drawDotted:(NSArray *)points pathClose:(BOOL)pathClose onLayer:(CALayer *)layer attributes:(DTLAttr)attributes;

// 设置path
- (void)setShapePath:(UIBezierPath *)shapePath;

// 修改path上的点
- (void)setPoints:(NSArray *)points;


@end
