//
//  CAShapeLayer+Other.m
//  Test
//
//  Created by temp on 16/8/11.
//  Copyright © 2016年 ehoo100. All rights reserved.
//

#import "CAShapeLayer+Other.h"
#import <objc/runtime.h>

static const void *kShapeLayerPath = @"kShapeLayerPath";
static const void *kShapeLayerPoints = @"kShapeLayerPoints";
static const void *kShapeLayerPathClose = @"kShapeLayerPathClose";

@implementation CAShapeLayer (Other)

+ (instancetype)drawLine:(NSArray *)points onLayer:(CALayer *)layer strokeColor:(CGColorRef)strokeColor lineWidth:(CGFloat)lineWidth {
    
    CAShapeLayer *shapeLayer = [[CAShapeLayer alloc] initWithPoints:points onLayer:layer lineWidth:lineWidth];
    shapeLayer.strokeColor = strokeColor;
    shapeLayer.fillColor = [UIColor clearColor].CGColor;
    [layer addSublayer:shapeLayer];
    return shapeLayer;
}

+ (instancetype)drawShape:(NSArray *)points onLayer:(CALayer *)layer fillColor:(CGColorRef)fillColor lineWidth:(CGFloat)lineWidth {
    
    CAShapeLayer *shapeLayer = [[CAShapeLayer alloc] initWithPoints:points onLayer:layer lineWidth:lineWidth];
    shapeLayer.fillColor = fillColor;
    shapeLayer.strokeColor = [UIColor clearColor].CGColor;
    [layer addSublayer:shapeLayer];
    return shapeLayer;
}

// patterns 包含四个元素：strokeColor（UIColor *), fillColor(UIColor *), lineWidth(CGFloat), lineLengthAndSpace(NSArray *:@[dottedLength, space])
+ (instancetype)drawDotted:(NSArray *)points pathClose:(BOOL)pathClose onLayer:(CALayer *)layer attributes:(DTLAttr)attributes {
    
    CAShapeLayer *shapeLayer = [[CAShapeLayer alloc] initWithPoints:points pathClose:pathClose onLayer:layer lineWidth:attributes.lineWidth];
    if (attributes.strokeColor) {
        shapeLayer.strokeColor = attributes.strokeColor;
    }
    if (attributes.fillColor) {
        shapeLayer.fillColor = attributes.fillColor;
    }
    if (attributes.dottedLength && attributes.dottedSpace) {
        shapeLayer.lineDashPattern = @[@(attributes.dottedLength), @(attributes.dottedSpace)];
    }
    [layer addSublayer:shapeLayer];
    return shapeLayer;
}

- (instancetype)initWithPoints:(NSArray *)points pathClose:(BOOL)pathClose onLayer:(CALayer *)layer lineWidth:(CGFloat)lineWidth {
    if (self = [super init]) {
        if (pathClose) {
            [self setPathClose:pathClose];
        }
        self.lineWidth = lineWidth;
        UIBezierPath *path = [UIBezierPath bezierPath];
        [self setShapePath:path];
        [self setPoints:points];
    }
    return self;
}

- (instancetype)initWithPoints:(NSArray *)points onLayer:(CALayer *)layer lineWidth:(CGFloat)lineWidth {
    
    if (self = [[CAShapeLayer alloc] initWithPoints:points pathClose:NO onLayer:layer lineWidth:lineWidth]){
    }
    return self;
}

#pragma mark - Setter&Getter
/* path */
- (void)setShapePath:(UIBezierPath *)shapePath {
    objc_setAssociatedObject(self, kShapeLayerPath, shapePath, OBJC_ASSOCIATION_COPY_NONATOMIC);
}

- (UIBezierPath *)shapePath {
    return objc_getAssociatedObject(self, kShapeLayerPath);
}

/* points */
- (void)setPoints:(NSArray *)points {
    if (!points || !points.count) {
        return;
    }
    objc_setAssociatedObject(self, kShapeLayerPoints, points, OBJC_ASSOCIATION_COPY_NONATOMIC);
    [self addPointsOnPath];
}

- (NSArray *)points {
    return objc_getAssociatedObject(self, kShapeLayerPoints);
}

/* closePath */
- (void)setPathClose:(BOOL)pathClose {
    objc_setAssociatedObject(self, kShapeLayerPathClose, @(pathClose), OBJC_ASSOCIATION_COPY_NONATOMIC);
}

- (BOOL)pathClose {
    return (BOOL)[objc_getAssociatedObject(self, kShapeLayerPathClose) integerValue];
}

#pragma mark - AddPoints
- (void)addPointsOnPath {
    
    UIBezierPath *path = [self shapePath];
    [path removeAllPoints];
    
    // 把表示坐标x,y的数组转化为CGPoint
    NSArray *points = [self points];
    [points enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        CGPoint point = CGPointZero;
        point.x = [obj[0] floatValue];
        point.y = [obj[1] floatValue];
        // 把point添加到path
        if (idx == 0) {
            [path moveToPoint:point];
        } else {
            [path addLineToPoint:point];
        }
    }];
    
    if ([self pathClose]) {
        [path closePath];
    }
    self.path = path.CGPath;
}


@end
