//
//  LGLHistogram.m
//  ServiceStation
//
//  Created by LGL on 2016/12/23.
//  Copyright © 2016年 机智的静默. All rights reserved.
//

#import "LGLHistogram.h"
#import "CAShapeLayer+Other.h"
#define HistogramColor(R,G,B,A) [UIColor colorWithRed:R/255.0 green:G/255.0 blue:B/255.0 alpha:A]
#define HistogramYellowColor HistogramColor(254,197,32, 1)
#define HistogramBlueColor HistogramColor(25, 186, 255, 1)
#define HistogramScreenScale  [UIScreen mainScreen].bounds.size.width/375.0

static const CGFloat HistogramLineWidth = 0.5;
static const CGFloat HistogramMarign = 20.0;
static const CGFloat HistogramArrowWidth = 12.0;// 包括宽高

@interface LGLHistogram ()
{
    CGFloat _histogramWidth;
    CGFloat _descLabelHeight;
}
@property (nonatomic, strong) UIView *histogramView;
@property (nonatomic, strong) UILabel *descLabel;
@end

@implementation LGLHistogram
- (instancetype)initWithFrame:(CGRect)frame saleAmount:(CGFloat)saleAmount costAmount:(CGFloat)costAmount scrapCost:(CGFloat)scrapCost {
    
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = [UIColor whiteColor];
        
        _histogramWidth = 80.0 * HistogramScreenScale;
        _descLabelHeight = scrapCost != 0 ? 40.0 : 0.0;
        self.saleAmount = saleAmount;
        self.costAmount = costAmount;
        self.scrapCost = scrapCost;
    }
    return self;
}

#pragma mark - 绘图方法
// 画直方图区域
- (void)drawBarGraph {
    [self drawAxis];
    [self drawSaleArea];
    [self drawCostArea];
}

// X、Y轴及直方图区域
- (UIView *)histogramView {
    if (!_histogramView) {
        _histogramView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.width, self.height - _descLabelHeight)];
        _histogramView.backgroundColor = [UIColor clearColor];
        _histogramView.clipsToBounds = NO;
        [self addSubview:_histogramView];
    }
    return _histogramView;
}

// 描述文本区域
- (UILabel *)descLabel {
    if (!_descLabel) {
        _descLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, self.height - _descLabelHeight, self.width, _descLabelHeight)];
        _descLabel.backgroundColor = [UIColor clearColor];
        _descLabel.textColor = [UIColor lightGrayColor];
        _descLabel.font = [UIFont systemFontOfSize:12.0];
        [self addSubview:_descLabel];
    }
    return _descLabel;
}

// 画X轴、Y轴
- (void)drawAxis {
    /* X轴 */
    // x轴的箭头
    NSArray *xArrowPoints = @[@[@(self.histogramView.width - HistogramMarign - HistogramArrowWidth), @(self.histogramView.height - HistogramArrowWidth * 1.5)], @[@(self.histogramView.width - HistogramMarign), @(self.histogramView.height - HistogramArrowWidth)], @[@(self.histogramView.width - HistogramMarign - HistogramArrowWidth), @(self.histogramView.height - HistogramArrowWidth * 0.5)]];
    [CAShapeLayer drawLine:xArrowPoints onLayer:self.histogramView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:HistogramLineWidth];
    // x轴上的线条
    NSArray *xAxisPoints = @[@[@(HistogramMarign - HistogramArrowWidth * 0.5), @(self.histogramView.height - HistogramArrowWidth)], xArrowPoints[1]];
    [CAShapeLayer drawLine:xAxisPoints onLayer:self.histogramView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:HistogramLineWidth];
    
    /* Y轴 */
    // y轴的箭头
    NSArray *yArrowPoints = @[@[@(HistogramMarign), @(HistogramMarign + HistogramArrowWidth)], @[@(HistogramMarign + HistogramArrowWidth * 0.5), @(HistogramMarign)], @[@(HistogramMarign + HistogramArrowWidth), @(HistogramMarign + HistogramArrowWidth)]];
    [CAShapeLayer drawLine:yArrowPoints onLayer:self.histogramView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:HistogramLineWidth];
    // y轴上的线条
    NSArray *yAxisPoints = @[yArrowPoints[1], @[@(HistogramMarign + HistogramArrowWidth * 0.5), @(self.histogramView.height)]];
    [CAShapeLayer drawLine:yAxisPoints onLayer:self.histogramView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:HistogramLineWidth];
    
    /* 坐标原点的“0” */
    UILabel *zeroLabel = [[UILabel alloc] initWithFrame:CGRectMake(HistogramMarign - HistogramArrowWidth * 0.5, self.histogramView.height - HistogramArrowWidth, HistogramArrowWidth, HistogramArrowWidth)];
    zeroLabel.font = [UIFont systemFontOfSize:10];
    zeroLabel.text = @"0";
    zeroLabel.textAlignment = NSTextAlignmentCenter;
    zeroLabel.textColor = [UIColor grayColor];
    [self.histogramView addSubview:zeroLabel];
}

// 画销售收入区域
- (void)drawSaleArea {
    // 直方图
    CGFloat costHeight = (self.histogramView.height - HistogramMarign - HistogramArrowWidth) * 0.85;
    CGFloat saleHeight = MIN(1, _saleAmount / _costAmount) * costHeight;
    NSArray *saleAreaPoints = @[@[@(_histogramWidth), @(self.histogramView.height - HistogramArrowWidth - saleHeight)], @[@(_histogramWidth * 2), @(self.histogramView.height - HistogramArrowWidth - saleHeight)], @[@(_histogramWidth * 2), @(self.histogramView.height - HistogramArrowWidth)], @[@(_histogramWidth), @(self.histogramView.height - HistogramArrowWidth)]];
    [CAShapeLayer drawShape:saleAreaPoints onLayer:self.histogramView.layer fillColor:HistogramYellowColor.CGColor lineWidth:HistogramLineWidth];
    // 销售收入
    UILabel *saleAmountLabel = [[UILabel alloc] initWithFrame:CGRectMake(_histogramWidth, self.histogramView.height - HistogramArrowWidth - saleHeight, _histogramWidth, saleHeight)];
    saleAmountLabel.font = [UIFont systemFontOfSize:13.0];
    saleAmountLabel.text = [self stringFromNumber:@(_saleAmount)];
    saleAmountLabel.textAlignment = NSTextAlignmentCenter;
    saleAmountLabel.textColor = [UIColor whiteColor];
    if (saleHeight < 13) {
        saleAmountLabel.frame = CGRectMake(saleAmountLabel.x, saleAmountLabel.y - 13, saleAmountLabel.width, 13);
        saleAmountLabel.textColor = [UIColor grayColor];
    }
    [self.histogramView addSubview:saleAmountLabel];
    // 商品销售收入标题
    UILabel *saleTitleLabel = [[UILabel alloc] initWithFrame:CGRectMake(_histogramWidth, self.histogramView.height - 5, _histogramWidth, 10)];
    saleTitleLabel.font = [UIFont systemFontOfSize:10.0];
    saleTitleLabel.text = @"商品销售收入";
    saleTitleLabel.textAlignment = NSTextAlignmentCenter;
    saleTitleLabel.textColor = HistogramYellowColor;
    [self.histogramView addSubview:saleTitleLabel];
}

// 画商品成本区域
- (void)drawCostArea {
    CGFloat costHeight = (self.histogramView.height - HistogramMarign - HistogramArrowWidth) * 0.85;
    NSArray *costAreaPoints =  @[@[@(_histogramWidth * 3), @(self.histogramView.height - HistogramArrowWidth - costHeight)], @[@(_histogramWidth * 4), @(self.histogramView.height - HistogramArrowWidth - costHeight)], @[@(_histogramWidth * 4), @(self.histogramView.height - HistogramArrowWidth)], @[@(_histogramWidth * 3), @(self.histogramView.height - HistogramArrowWidth)]];
    [CAShapeLayer drawShape:costAreaPoints onLayer:self.histogramView.layer fillColor:HistogramBlueColor.CGColor lineWidth:HistogramLineWidth];
    // 商品成本
    UILabel *costAmountLabel = [[UILabel alloc] initWithFrame:CGRectMake(_histogramWidth * 3, self.histogramView.height - HistogramArrowWidth - costHeight, _histogramWidth, costHeight)];
    costAmountLabel.font = [UIFont systemFontOfSize:13.0];
    costAmountLabel.text = [self stringFromNumber:@(_costAmount)];
    costAmountLabel.textAlignment = NSTextAlignmentCenter;
    costAmountLabel.textColor = [UIColor whiteColor];
    [self.histogramView addSubview:costAmountLabel];
    // 商品成本支出标题
    UILabel *costitleLabel = [[UILabel alloc] initWithFrame:CGRectMake(_histogramWidth * 3, self.histogramView.height - 5, _histogramWidth, 10)];
    costitleLabel.font = [UIFont systemFontOfSize:10.0];
    costitleLabel.text = @"商品成本支出";
    costitleLabel.textAlignment = NSTextAlignmentCenter;
    costitleLabel.textColor = HistogramBlueColor;
    [self.histogramView addSubview:costitleLabel];
}

#pragma mark - Setter
- (void)setSaleAmount:(CGFloat)saleAmount {
    _saleAmount = saleAmount;
    if (_costAmount) {
        [self drawBarGraph];
    }
}

- (void)setCostAmount:(CGFloat)costAmount {
    _costAmount = costAmount;
    if (_saleAmount) {
        [self drawBarGraph];
    }
}

- (void)setScrapCost:(CGFloat)scrapCost {
    _scrapCost = scrapCost;
    if (scrapCost != 0) {
        self.descLabel.text = [NSString stringWithFormat:@"      * 今日报损成本支出   %@   ", [self stringFromNumber:@(_scrapCost)]];
    }
}
#pragma mark - 工具方法
// 数字转汉字，最多保留两位，大于小数点后两位的一律舍去
- (NSString *)stringFromNumber:(NSNumber *)number {
    
    NSString *string = [NSString stringWithFormat:@"%.2f", [number floatValue]];
    string = [NSString stringWithFormat:@"%@", @([string floatValue])];
    
    return string;
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
