//
//  LGLHistogramView.m
//  Test
//
//  Created by LGL on 2016/12/23.
//  Copyright © 2016年 ehoo100. All rights reserved.
//

#import "LGLHistogramView.h"
#import "LGLChartUtil.h"

// 头部利润容器视图的高度
static const CGFloat headerHeight = 40.0;
// 直方图底部边距
static CGFloat histogramBottomMargin = 48.0;
// 底部分割线的高度
static const CGFloat separatorLineHeight = 4.5;
// 直方图的全部高度
static NSString *const LGLHistogramFullHeight = @"LGLHistogramFullHeight";
// 直方图的实体高度
static NSString *const LGLHistogramEntityHeight = @"LGLHistogramEntityHeight";

@interface LGLHistogramView ()
{
    // 直方图宽度
    CGFloat _histogramWidth;
    // 直方图的最大高度
    CGFloat _maxHistogramHeight;
    // 底部报损成本的高度
    CGFloat _scrapLabelHeight;
}
// 头部利润、图例容器视图
@property (nonatomic, strong) UIView *headerView;
// 今日利润合计数量
@property (nonatomic, strong) UILabel *todayProfitLabel;
// 报损合计
@property (nonatomic, strong) UILabel *scrapLabel;

/* 直方图 */
// 直方图容器视图
@property (nonatomic, strong) UIView *histogramView;
// 直方图图例的容器视图
@property (nonatomic, strong) UIView *histogramLegend;
// 销售直方图
@property (nonatomic, strong) CAShapeLayer *saleHistogram;
// 销售额
@property (nonatomic, strong) UILabel *saleLabel;
// 亏损虚线
@property (nonatomic, strong) CAShapeLayer *deficitDotted;
// 亏损额
@property (nonatomic, strong) UILabel *deficitLabel;
// 成本直方图
@property (nonatomic, strong) CAShapeLayer *costHistogram;
// 成本
@property (nonatomic, strong) UILabel *costLabel;
// 利润虚线
@property (nonatomic, strong) CAShapeLayer *profitDotted;
// 利润额
@property (nonatomic, strong) UILabel *profitLabel;


@end

@implementation LGLHistogramView

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = LGLChartBackgroundColor;
        _histogramWidth = 74.0 * LGLChartScreenScale;
        _maxHistogramHeight = 158.0;
        _scrapLabelHeight = 37.5;
        _saleAmount = -1;
        _costAmount = -1;
    }
    return self;
}

- (instancetype)initWithFrame:(CGRect)frame dayProfit:(CGFloat)dayProfit saleAmount:(CGFloat)saleAmount costAmount:(CGFloat)costAmount scrapCost:(CGFloat)scrapCost {
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = LGLChartBackgroundColor;
        _histogramWidth = 74.0 * LGLChartScreenScale;
        _maxHistogramHeight = 158.0;
        _scrapLabelHeight = 37.5;
        _saleAmount = -1;
        _costAmount = -1;
        
        self.dayProfit = dayProfit;
        self.saleAmount = saleAmount;
        self.costAmount = costAmount;
        self.scrapCost = scrapCost;
    }
    return self;
}

#pragma mark - 画图方法
- (void)drawHistogram {
    
    [self drawSaleArea];
    [self drawCostArea];
}

// 头视图
- (UIView *)headerView {
    if (!_headerView) {
        _headerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.width, headerHeight)];
        _headerView.backgroundColor = [UIColor whiteColor];
        [self addSubview:_headerView];
        
        // 标题
        UIButton *profixTitle = [[UIButton alloc] initWithFrame:CGRectMake(10, 10, 135, 20)];
        profixTitle.titleLabel.font = [UIFont systemFontOfSize:15.0];
        [profixTitle setTitle:@"今日毛利润合计" forState:UIControlStateNormal];
        [profixTitle setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [profixTitle setImage:[UIImage imageNamed:@"money"] forState:UIControlStateNormal];
        profixTitle.titleEdgeInsets = UIEdgeInsetsMake(0, 5, 0, -5);
        [_headerView addSubview:profixTitle];
        
        // 利润(见- (UILabel *)profitLabel)
        
        // 分割线
        UIView *separator = [[UIView alloc] initWithFrame:CGRectMake(0, headerHeight - LGLChartLineWidth, _headerView.width, LGLChartLineWidth)];
        separator.backgroundColor = LGLChartColor(222, 222, 222, 1);
        [_headerView addSubview:separator];
    }
    return _headerView;
}

// 今日利润合计
- (UILabel *)todayProfitLabel {
    if (!_todayProfitLabel) {
       
        _todayProfitLabel = [[UILabel alloc] initWithFrame:CGRectMake(150, 10, self.headerView.width - 158, 20)];
        _todayProfitLabel.font = [UIFont systemFontOfSize:17.0];
        _todayProfitLabel.textColor = LGLChartYellowColor;
        _todayProfitLabel.textAlignment = NSTextAlignmentRight;
        [self.headerView addSubview:_todayProfitLabel];
    }
    return _todayProfitLabel;
}

// 今日报损支出区域
- (UILabel *)scrapLabel {
    if (!_scrapLabel) {
        _scrapLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, self.height - _scrapLabelHeight - separatorLineHeight, self.width, _scrapLabelHeight)];
        _scrapLabel.backgroundColor = [UIColor whiteColor];
        _scrapLabel.textColor = [UIColor blackColor];
        _scrapLabel.font = [UIFont systemFontOfSize:15.0];
        [self addSubview:_scrapLabel];
    } else {
        _scrapLabel.frame = CGRectMake(0, self.height - _scrapLabelHeight - separatorLineHeight, self.width, _scrapLabelHeight);
    }
    return _scrapLabel;
}

// 直方图容器
- (UIView *)histogramView {
    if (!_histogramView) {
        _histogramView = [[UIView alloc] initWithFrame:CGRectMake(0, self.headerView.height, self.width, self.height - self.headerView.height - self.scrapLabel.height - separatorLineHeight)];
        _histogramView.backgroundColor = [UIColor whiteColor];
        [self addSubview:_histogramView];
        
        // 图例
        [self.histogramView addSubview:self.histogramLegend];
    }
    return _histogramView;
}

// 销售直方图
- (CAShapeLayer *)saleHistogram {
    if (!_saleHistogram) {
        _saleHistogram = [CAShapeLayer drawShape:nil onLayer:self.histogramView.layer fillColor:LGLChartYellowColor.CGColor lineWidth:LGLChartLineWidth];
    }
    return _saleHistogram;
}

// 亏损虚线
- (CAShapeLayer *)deficitDotted {
    if (!_deficitDotted) {
        _deficitDotted = [CAShapeLayer drawDotted:nil pathClose:YES onLayer:self.histogramView.layer attributes:DTLAttrMake([UIColor blackColor].CGColor, [UIColor clearColor].CGColor, LGLChartLineWidth, 1, 2)];
    }
    return _deficitDotted;
}

// 成本直方图
- (CAShapeLayer *)costHistogram {
    if (!_costHistogram) {
        _costHistogram = [CAShapeLayer drawShape:nil onLayer:self.histogramView.layer fillColor:LGLChartBlueColor.CGColor lineWidth:LGLChartLineWidth];
    }
    return _costHistogram;
}

// 利润虚线
- (CAShapeLayer *)profitDotted {
    if (!_profitDotted) {
        _profitDotted = [CAShapeLayer drawDotted:nil pathClose:YES onLayer:self.histogramView.layer attributes:DTLAttrMake([UIColor blackColor].CGColor, [UIColor clearColor].CGColor, LGLChartLineWidth, 1, 2)];
    }
    return _profitDotted;
}

// 销售额
- (UILabel *)saleLabel {
    if (!_saleLabel) {
        _saleLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _saleLabel.backgroundColor = [UIColor clearColor];
        _saleLabel.textColor = [UIColor blackColor];
        _saleLabel.textAlignment = NSTextAlignmentCenter;
        [self.histogramView addSubview:_saleLabel];
    }
    return _saleLabel;
}

// 亏损额
- (UILabel *)deficitLabel {
    if (!_deficitLabel) {
        _deficitLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _deficitLabel.backgroundColor = [UIColor clearColor];
        _deficitLabel.textColor = [UIColor blackColor];
        _deficitLabel.textAlignment = NSTextAlignmentCenter;
        _deficitLabel.hidden = YES;// 默认隐藏
        [self.histogramView addSubview:_deficitLabel];
    }
    return _deficitLabel;
}

// 成本
- (UILabel *)costLabel {
    if (!_costLabel) {
        _costLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _costLabel.backgroundColor = [UIColor clearColor];
        _costLabel.textColor = [UIColor blackColor];
        _costLabel.textAlignment = NSTextAlignmentCenter;
        [self.histogramView addSubview:_costLabel];
    }
    return _costLabel;
}

// 利润额
- (UILabel *)profitLabel {
    if (!_profitLabel) {
        _profitLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _profitLabel.backgroundColor = [UIColor clearColor];
        _profitLabel.textColor = [UIColor blackColor];
        _profitLabel.textAlignment = NSTextAlignmentCenter;
        _profitLabel.hidden = YES;// 默认隐藏
        [self.histogramView addSubview:_profitLabel];
    }
    return _profitLabel;
}

// 直方图图例容器
- (UIView *)histogramLegend {
    if (!_histogramLegend) {
        _histogramLegend = [[UIView alloc] initWithFrame:CGRectMake(0, 202, self.histogramView.width, 16 * LGLChartScreenScale)];
        _histogramLegend.backgroundColor = [UIColor clearColor];
        
        [self drawChartLegend];
    } else {
        
        if (_scrapLabelHeight == 0) {
            _histogramLegend.frame = CGRectMake(0, 230, self.histogramView.width, 16 * LGLChartScreenScale);
        } else {
            _histogramLegend.frame = CGRectMake(0, 202, self.histogramView.width, 16 * LGLChartScreenScale);
        }
    }
    return _histogramLegend;
}

// 直方图图例
- (void)drawChartLegend {
    /* 利润标题 */
    // 虚线框
    NSArray *dottedPoints = @[@[@(15 * LGLChartScreenScale), @0], @[@(31 * LGLChartScreenScale), @0], @[@(31 * LGLChartScreenScale), @(16 * LGLChartScreenScale)], @[@(15 * LGLChartScreenScale), @(16 * LGLChartScreenScale)]];
    [CAShapeLayer drawDotted:dottedPoints pathClose:YES onLayer:self.histogramLegend.layer attributes:DTLAttrMake([UIColor blackColor].CGColor, [UIColor clearColor].CGColor, LGLChartLineWidth, 1, 1)];
    
    // 利润
    UILabel *profitLabel = [[UILabel alloc] initWithFrame:CGRectMake(43 * LGLChartScreenScale, 0, 28 * LGLChartScreenScale, 13 * LGLChartScreenScale)];
    profitLabel.font = [UIFont systemFontOfSize:13 * LGLChartScreenScale];
    profitLabel.text = @"利润";
    profitLabel.textColor = [UIColor darkGrayColor];
    [self.histogramLegend addSubview:profitLabel];
    
    /* 商品销售收入标题 */
    // 方块
    NSArray *salePoints = @[@[@(102 * LGLChartScreenScale), @0], @[@(118 * LGLChartScreenScale), @0], @[@(118 * LGLChartScreenScale), @(16 * LGLChartScreenScale)], @[@(102 * LGLChartScreenScale), @(16 * LGLChartScreenScale)]];
    [CAShapeLayer drawShape:salePoints onLayer:self.histogramLegend.layer fillColor:LGLChartYellowColor.CGColor lineWidth:LGLChartLineWidth];
    // 商品销售收入
    UILabel *saleTitleLabel = [[UILabel alloc] initWithFrame:CGRectMake(130 * LGLChartScreenScale, 0, 80 * LGLChartScreenScale, 13 * LGLChartScreenScale)];
    saleTitleLabel.font = [UIFont systemFontOfSize:13.0 * LGLChartScreenScale];
    saleTitleLabel.text = @"商品销售收入";
    saleTitleLabel.textColor = [UIColor darkGrayColor];
    [self.histogramLegend addSubview:saleTitleLabel];
    
    /* 商品成本支出标题 */
    // 方块
    NSArray *costPoints = @[@[@(231 * LGLChartScreenScale), @0], @[@(247 * LGLChartScreenScale), @0], @[@(247 * LGLChartScreenScale), @(16 * LGLChartScreenScale)], @[@(231 * LGLChartScreenScale), @(16 * LGLChartScreenScale)]];
    [CAShapeLayer drawShape:costPoints onLayer:self.histogramLegend.layer fillColor:LGLChartBlueColor.CGColor lineWidth:LGLChartLineWidth];
    // 商品成本支出
    UILabel *costitleLabel = [[UILabel alloc] initWithFrame:CGRectMake(258 * LGLChartScreenScale, 0, 80 * LGLChartScreenScale, 13 * LGLChartScreenScale)];
    costitleLabel.font = [UIFont systemFontOfSize:13.0 * LGLChartScreenScale];
    costitleLabel.text = @"商品成本支出";
    costitleLabel.textColor = [UIColor darkGrayColor];
    [self.histogramLegend addSubview:costitleLabel];
}

// 画销售收入区域
- (void)drawSaleArea {
    NSDictionary *heights = [self getFullAndEntityHeightForSale:YES];
    [self drawHistogramWithFullHeight:[heights[LGLHistogramFullHeight] doubleValue] EntityHeight:[heights[LGLHistogramEntityHeight] doubleValue] isSale:YES];
}

// 画商品成本区域
- (void)drawCostArea {
    NSDictionary *heights = [self getFullAndEntityHeightForSale:NO];
    [self drawHistogramWithFullHeight:[heights[LGLHistogramFullHeight] doubleValue] EntityHeight:[heights[LGLHistogramEntityHeight] doubleValue] isSale:NO];
}

// 画直方图（虚线、实线区域）
- (void)drawHistogramWithFullHeight:(CGFloat)fullHeight EntityHeight:(CGFloat)entityHeight isSale:(BOOL)isSale {
    // 亏损或盈利
    CGFloat deficitOrProfit = _saleAmount - _costAmount;
    // 左边距
    CGFloat leftMargin = isSale ? 84.0 * LGLChartScreenScale : 215.0 * LGLChartScreenScale;
    // 亏损或盈利的高度
    CGFloat dottedHeight = fullHeight - entityHeight;
    // 标记变量：是否都为零
    BOOL bothZero = NO;
    if (_saleAmount == 0 && _costAmount == 0) {
        bothZero = YES;
    }
    // 标记变量：是否为0
    BOOL valueZero = NO;
    if ((isSale && _saleAmount == 0) || (!isSale && _costAmount == 0)) {
        valueZero = YES;
    }
    // 盈利或亏损虚线框
    if (dottedHeight == 0) {
        // 隐藏虚线框
        if (isSale) {
            self.deficitDotted.strokeColor = [UIColor clearColor].CGColor;
        } else {
            self.profitDotted.strokeColor = [UIColor clearColor].CGColor;
        }
    } else {
        NSArray *dottedPoints = @[@[@(leftMargin + LGLChartLineWidth), @(self.histogramView.height - histogramBottomMargin - fullHeight)], @[@(leftMargin + _histogramWidth - LGLChartLineWidth), @(self.histogramView.height - histogramBottomMargin - fullHeight)], @[@(leftMargin + _histogramWidth - LGLChartLineWidth), @(self.histogramView.height - histogramBottomMargin - entityHeight)], @[@(leftMargin + LGLChartLineWidth), @(self.histogramView.height - histogramBottomMargin - entityHeight)]];
        if (isSale) {
            [self.deficitDotted setPoints:dottedPoints];
            self.deficitDotted.strokeColor = [UIColor blackColor].CGColor;
        } else {
            [self.profitDotted setPoints:dottedPoints];
            self.profitDotted.strokeColor = [UIColor blackColor].CGColor;
        }
    }
    
    // 盈利或亏损数量的坐标
    CGRect rect = CGRectMake(leftMargin + LGLChartLineWidth, self.histogramView.height - histogramBottomMargin - fullHeight, _histogramWidth - LGLChartLineWidth * 2, dottedHeight);
    if (dottedHeight < 13) {
        rect = CGRectMake(leftMargin + LGLChartLineWidth, self.histogramView.height - histogramBottomMargin - fullHeight - 15, _histogramWidth - LGLChartLineWidth * 2, 13);
    }
    
    // 实体直方图的点
    NSArray *histogramPoints = @[@[@(leftMargin), @(self.histogramView.height - histogramBottomMargin - entityHeight + LGLChartLineWidth)], @[@(leftMargin + _histogramWidth), @(self.histogramView.height - histogramBottomMargin - entityHeight - LGLChartLineWidth)], @[@(leftMargin + _histogramWidth), @(self.histogramView.height - histogramBottomMargin)], @[@(leftMargin), @(self.histogramView.height - histogramBottomMargin)]];
    
    // 实体直方图上的数量的坐标
    CGFloat startY = self.histogramView.height - histogramBottomMargin - entityHeight - LGLChartLineWidth;
    CGRect rect2 = CGRectMake(leftMargin, startY, _histogramWidth, entityHeight);
    if (entityHeight < 13) {
        rect2 = CGRectMake(leftMargin, startY + entityHeight + 2, _histogramWidth, 13);
    }
    // 显示成本或销售、利润或亏损
    if (isSale) {
        // 销售
        [self.saleHistogram setPoints:histogramPoints];
        self.saleHistogram.fillColor = LGLChartYellowColor.CGColor;
        if (bothZero) {
            self.saleHistogram.fillColor = LGLChartLightGrayColor.CGColor;
        } else if (valueZero) {
            self.saleHistogram.fillColor = [UIColor clearColor].CGColor;
        }
        self.saleLabel.frame = rect2;
        self.saleLabel.text = [LGLChartUtil stringFromNumber:@(_saleAmount)];
        self.deficitLabel.frame = rect;
        // 亏损
        if (deficitOrProfit < 0) {
            self.deficitLabel.hidden = NO;
            self.deficitLabel.text = [LGLChartUtil stringFromNumber:@(deficitOrProfit)];
        } else {
            self.deficitLabel.hidden = YES;
        }
    } else {
        // 成本
        [self.costHistogram setPoints:histogramPoints];
        self.costHistogram.fillColor = LGLChartBlueColor.CGColor;
        if (bothZero) {
            self.costHistogram.fillColor = LGLChartLightGrayColor.CGColor;
        } else if (valueZero) {
            self.costHistogram.fillColor = [UIColor clearColor].CGColor;
        }
        self.costLabel.frame = rect2;
        self.costLabel.text = [LGLChartUtil stringFromNumber:@(_costAmount)];
        // 利润
        self.profitLabel.frame = rect;
        if (deficitOrProfit > 0) {
            self.profitLabel.hidden = NO;
            self.profitLabel.text = [LGLChartUtil stringFromNumber:@(deficitOrProfit)];
        } else {
            if (valueZero) {
                self.profitLabel.hidden = NO;
                self.profitLabel.text = [LGLChartUtil stringFromNumber:@0];
            } else {
                self.profitLabel.hidden = YES;
            }
        }
    }
}

#pragma mark - Setter
- (void)setDayProfit:(CGFloat)dayProfit {
    _dayProfit = dayProfit;
    self.todayProfitLabel.text = [LGLChartUtil stringFromNumber:@(_dayProfit)];
}

- (void)setSaleAmount:(CGFloat)saleAmount {
    if (saleAmount == _saleAmount) {
        return;
    }
    _saleAmount = saleAmount;
//    if (_costAmount >= 0) {
//        [self drawHistogram];
//    }
}

- (void)setCostAmount:(CGFloat)costAmount {
    if (costAmount == _costAmount) {
        return;
    }
    _costAmount = costAmount;
//    if (_saleAmount >= 0) {
//        [self drawHistogram];
//    }
}

- (void)setScrapCost:(CGFloat)scrapCost {
    _scrapCost = scrapCost;
    if (scrapCost != 0) {
        _maxHistogramHeight = 158.0;
        _scrapLabelHeight = 37.5;
    } else {
        _maxHistogramHeight = 196.0;
        _scrapLabelHeight = 0;
    }
    self.scrapLabel.text = [NSString stringWithFormat:@"      * 今日报损成本支出   %@   ", [LGLChartUtil stringFromNumber:@(-_scrapCost)]];
    [self drawHistogram];
}

#pragma mark - 工具方法
// 获取直方图的全部高度和实体高度，返回参数是字典，有两个键值：fullHeight、entityHeight
- (NSDictionary *)getFullAndEntityHeightForSale:(BOOL)forSale {
    
    // 全部高度
    CGFloat fullValue = MAX(LGLChartZero, MAX(_saleAmount, _costAmount));
    CGFloat fullHeight = _maxHistogramHeight;
    
    // 实体高度
    CGFloat entityValue = MAX(LGLChartZero, forSale ? _saleAmount : _costAmount);
    CGFloat entityHeight = entityValue / fullValue * fullHeight;
    if (_saleAmount == 0 && _costAmount == 0) {
        entityHeight = forSale ? 0 : fullHeight/2;
    }
    
    return @{@"LGLHistogramFullHeight": @(fullHeight), @"LGLHistogramEntityHeight": @(entityHeight)};
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
