//
//  LGLLineChart.m
//  Test
//
//  Created by LGL on 2016/12/21.
//  Copyright © 2016年 罗桂林. All rights reserved.
//

#import "LGLLineChart.h"
#import "CAShapeLayer+Other.h"
#define LineColor(R,G,B,A) [UIColor colorWithRed:R/255.0 green:G/255.0 blue:B/255.0 alpha:A]
#define LineChartYellowColor LineColor(254,197,32, 1)
#define LineChartBlueColor LineColor(25, 186, 255, 1)
#define LineChartScreenScale  [UIScreen mainScreen].bounds.size.width/375.0

static const CGFloat LineChartLineWidth = 0.5;
static const CGFloat LineChartMarign = 20.0;
static const CGFloat LineChartArrowWidth = 12.0;// 包括宽高

@interface LGLLineChart ()
{
    // 每个大间隔的宽度
    CGFloat _lineChartWidth;
    // 底部报损成本的高度
    CGFloat _descLabelHeight;
}
@property (nonatomic, strong) UIView *chartView;
@property (nonatomic, strong) UILabel *descLabel;
// x轴的y坐标
@property (nonatomic, assign) CGFloat xAxisPointY;

@end

@implementation LGLLineChart

- (instancetype)initWithFrame:(CGRect)frame saleAmounts:(NSArray *)saleAmounts grossProfits:(NSArray *)grossProfits scrapCost:(CGFloat)scrapCost {
    
    if (self = [super initWithFrame:frame]) {
        _lineChartWidth = 42.0 * LineChartScreenScale;
        _descLabelHeight = scrapCost != 0 ? 40.0 : 0.0;
        
        self.saleAmounts = saleAmounts;
        self.grossProfits = grossProfits;
        self.scrapCost = scrapCost;
    }
    return self;
}

#pragma mark - 绘图方法
// 画折线图区域
- (void)drawLineChart {
    [self drawAxis];
    [self drawDaysLabel];
    [self drawSaleChart];
    [self drawProfitChart];
    [self drawDottedLines];
}

// X、Y轴及折线图区域
- (UIView *)chartView {
    if (!_chartView) {
        _chartView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.width, self.height - _descLabelHeight)];
        _chartView.backgroundColor = [UIColor clearColor];
        _chartView.clipsToBounds = NO;
        [self addSubview:_chartView];
    }
    return _chartView;
}

// 描述文本区域
- (UILabel *)descLabel {
    if (!_descLabel) {
        _descLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, self.height - _descLabelHeight, self.width, _descLabelHeight)];
        _descLabel.backgroundColor = [UIColor clearColor];
        _descLabel.textColor = [UIColor lightGrayColor];
        _descLabel.font = [UIFont systemFontOfSize:12.0];
        [self addSubview:_descLabel];
    }
    return _descLabel;
}

// 画X轴、Y轴
- (void)drawAxis {
    /* X轴 */
    // x轴的箭头
    NSArray *xArrowPoints = @[@[@(self.chartView.width - LineChartMarign - LineChartArrowWidth), @(self.xAxisPointY - LineChartArrowWidth * 0.5)], @[@(self.chartView.width - LineChartMarign), @(self.xAxisPointY)], @[@(self.chartView.width - LineChartMarign - LineChartArrowWidth), @(self.xAxisPointY + LineChartArrowWidth * 0.5)]];
    [CAShapeLayer drawLine:xArrowPoints onLayer:self.chartView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:LineChartLineWidth];
    // x轴上的线条
    NSArray *xAxisPoints = @[@[@(LineChartMarign - LineChartArrowWidth * 0.5), @(self.xAxisPointY)], xArrowPoints[1]];
    [CAShapeLayer drawLine:xAxisPoints onLayer:self.chartView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:LineChartLineWidth];
    
    /* Y轴 */
    // y轴的箭头
    NSArray *yArrowPoints = @[@[@(LineChartMarign), @(LineChartMarign + LineChartArrowWidth)], @[@(LineChartMarign + LineChartArrowWidth * 0.5), @(LineChartMarign)], @[@(LineChartMarign + LineChartArrowWidth), @(LineChartMarign + LineChartArrowWidth)]];
    [CAShapeLayer drawLine:yArrowPoints onLayer:self.chartView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:LineChartLineWidth];
    // y轴上的线条
    NSArray *yAxisPoints = @[yArrowPoints[1], @[@(LineChartMarign + LineChartArrowWidth * 0.5), @(self.chartView.height- LineChartArrowWidth)]];
    [CAShapeLayer drawLine:yAxisPoints onLayer:self.chartView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:LineChartLineWidth];
    
    /* 右上角图例 */
    // 每日销售实收
    NSArray *colors = @[LineChartYellowColor, LineChartBlueColor];
    NSArray *textes = @[@"每日销售实收", @"每日毛利润"];
    for (NSInteger i = 0; i < 2; i ++) {
        // 圆点
        UIView *spot = [[UIView alloc] initWithFrame:CGRectMake(self.chartView.width - 80, 10 + 15 * i, 10, 10)];
        spot.backgroundColor = colors[i];
        spot.layer.cornerRadius = spot.width/2;
        spot.layer.masksToBounds = YES;
        [self.chartView addSubview:spot];
        // 文本
        UILabel *content = [[UILabel alloc] initWithFrame:CGRectMake(self.chartView.width - 65, spot.y, 65, 10)];
        content.font = [UIFont systemFontOfSize:10.0];
        content.text = textes[i];
        content.textColor = [UIColor grayColor];
        [self.chartView addSubview:content];
    }
}

// 画X轴上的日期
- (void)drawDaysLabel {
    NSUInteger count = _saleAmounts.count/4 + 1;
    NSArray *days = @[@"1日", @"5日", @"9日", @"13日", @"17日", @"21日", @"25日", @"29日"];
    for (NSUInteger i = 0; i < count; i ++) {
        UILabel *dayLabel = [[UILabel alloc] initWithFrame:CGRectMake(LineChartMarign + _lineChartWidth * i, self.chartView.height - LineChartArrowWidth, LineChartArrowWidth * 2, LineChartArrowWidth)];
        dayLabel.font = [UIFont systemFontOfSize:10];
        dayLabel.text = days[i];
        dayLabel.textColor = [UIColor grayColor];
        dayLabel.textAlignment = NSTextAlignmentCenter;
        [self.chartView addSubview:dayLabel];
    }
}

// 画每日销售的折线图
- (void)drawSaleChart {
    NSArray *allPoints = [self calculatePoints:_saleAmounts isSale:YES];
    [CAShapeLayer drawLine:allPoints onLayer:self.chartView.layer strokeColor:LineChartYellowColor.CGColor lineWidth:LineChartLineWidth];
    // 画点
    for (NSArray *point in allPoints) {
        CAShapeLayer *spot = [CAShapeLayer layer];
        spot.fillColor = LineChartYellowColor.CGColor;
        CGPoint center = CGPointMake([point[0] doubleValue], [point[1] doubleValue]);
        UIBezierPath *path = [UIBezierPath bezierPathWithArcCenter:center radius:3.0 startAngle:0 endAngle:M_PI * 2 clockwise:YES];;
        spot.path = path.CGPath;
        [self.chartView.layer addSublayer:spot];
    }
}

// 画每日毛利润的折线图
- (void)drawProfitChart {
    NSArray *allPoints = [self calculatePoints:_grossProfits isSale:NO];
    [CAShapeLayer drawLine:allPoints onLayer:self.chartView.layer strokeColor:LineChartBlueColor.CGColor lineWidth:LineChartLineWidth];
    // 画点
    for (NSArray *point in allPoints) {
        CAShapeLayer *spot = [CAShapeLayer layer];
        spot.fillColor = LineChartBlueColor.CGColor;
        CGPoint center = CGPointMake([point[0] doubleValue], [point[1] doubleValue]);
        UIBezierPath *path = [UIBezierPath bezierPathWithArcCenter:center radius:3.0 startAngle:0 endAngle:M_PI * 2 clockwise:YES];;
        spot.path = path.CGPath;
        [self.chartView.layer addSublayer:spot];
    }
}

// 画虚线
- (void)drawDottedLines {
    NSArray *salePoints = [self calculatePoints:_saleAmounts isSale:YES];
    NSArray *profitPoints = [self calculatePoints:_grossProfits isSale:NO];
    for (NSUInteger i = 1; i <= (_saleAmounts.count - 1)/4; i ++) {
        NSArray *salePoint = salePoints[i * 4];
        NSArray *profitPoint = profitPoints[i * 4];
        CGFloat pointX = [salePoint[0] doubleValue];
        CGFloat bottomPointY = _xAxisPointY;
        CGFloat topPointY = [salePoint[1] doubleValue];
        if (topPointY > [profitPoint[1] doubleValue]) {
            topPointY = [profitPoint[1] doubleValue];
        }
        // 画虚线
        // 虚线path
        UIBezierPath *path = [UIBezierPath bezierPath];
        [path moveToPoint:CGPointMake(pointX, topPointY)];
        [path addLineToPoint:CGPointMake(pointX, bottomPointY)];
        // 虚线图层
        CAShapeLayer *dotted = [CAShapeLayer layer];
        dotted.lineWidth = LineChartLineWidth;
        dotted.strokeColor = [UIColor lightGrayColor].CGColor;
        dotted.lineJoin = kCALineJoinRound;
        dotted.lineCap = kCALineCapRound;
        dotted.path = path.CGPath;
        dotted.lineDashPattern = @[@8, @4];// index0：线段长度，index1：间隔距离
        [self.chartView.layer addSublayer:dotted];
    }
}

#pragma markt - Setter
- (void)setSaleAmounts:(NSArray *)saleAmounts {
    _saleAmounts = saleAmounts;
    if (_grossProfits.count) {
        [self drawLineChart];
    }
}

- (void)setGrossProfits:(NSArray *)grossProfits {
    _grossProfits = grossProfits;
    if (_saleAmounts.count) {
        [self drawLineChart];
    }
}

- (void)setScrapCost:(CGFloat)scrapCost {
    _scrapCost = scrapCost;
    if (scrapCost != 0) {
        self.descLabel.text = [NSString stringWithFormat:@"      * 月度报损成本支出合计   %@   ", [self stringFromNumber:@(_scrapCost)]];
    }
}

#pragma mark - Getter
- (CGFloat)xAxisPointY {
    if (!_xAxisPointY) {
        _xAxisPointY = self.chartView.height - LineChartArrowWidth;
        if ([self hasNegativeProfits]) {
            
        }
    }
    return _xAxisPointY;
}

#pragma mark - 工具方法
// 利润是否有负的
- (BOOL)hasNegativeProfits {
    
    for (NSNumber *num in _grossProfits) {
        if ([num doubleValue] < 0.0) {
            return YES;
        }
    }
    return NO;
}

// 数字转汉字，最多保留两位，大于小数点后两位的一律舍去
- (NSString *)stringFromNumber:(NSNumber *)number {
    
    NSString *string = [NSString stringWithFormat:@"%.2f", [number floatValue]];
    string = [NSString stringWithFormat:@"%@", @([string floatValue])];
    
    return string;
}

// 计算出数组中数据对应的所有的点，isSale：标记变量，是销售额还是毛利润
- (NSArray *)calculatePoints:(NSArray *)datas isSale:(BOOL)isSale {
    NSMutableArray *points = [NSMutableArray array];
    // 最大高度，所有高度以此为基准
    CGFloat maxHeight = (self.chartView.height - LineChartMarign - LineChartArrowWidth) * 0.85;
    if (!isSale) {
        // 是计算毛利润的店
        maxHeight *= 0.6;
    }
    // 最大数值，任意数值num的y值：num／maxNum * maxHeight
    CGFloat maxNum = [self maxNumber:datas];
    
    for (NSUInteger i = 0; i < datas.count; i ++) {
        CGFloat x = LineChartMarign + LineChartArrowWidth/2 + i * _lineChartWidth/4;
        CGFloat y = self.chartView.height - LineChartArrowWidth - [datas[i] floatValue] / maxNum * maxHeight;
        [points addObject:@[@(x), @(y)]];
    }
    
    return [points copy];
}

- (CGFloat)maxNumber:(NSArray *)datas {
    CGFloat max = [datas[0] doubleValue];
    for (NSNumber *num in datas) {
        if (max < [num doubleValue]) {
            max = [num doubleValue];
        }
    }
    return max;
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
