//
//  LGLLineChartView.m
//  Test
//
//  Created by LGL on 2016/12/23.
//  Copyright © 2016年 ehoo100. All rights reserved.
//

#import "LGLLineChartView.h"
#import "LGLChartUtil.h"

// 头部利润、图例容器视图的高度
static const CGFloat headerHeight = 52.5;
// 底部分割线的高度
static const CGFloat separatorLineHeight = 6.5;
// 坐标上圆点的半径
static const CGFloat axisSpotRadius = 2.0;
// 折线上圆点的半径
static const CGFloat lineSpotRadius = 3.0;
// 坐标轴的边距
static const CGFloat axisMargin = 15.0;
// 日期label的宽度
static const CGFloat daysLabelWidth = 30.0;
// 日期label的高度
static const CGFloat daysLabelHeight = 20.0;
// 最高销售或最低利润label的高度
static const CGFloat maxOrMinAmountLabelHeight = 12.0;

@interface LGLLineChartView ()
{
    // 每个日期间隔的宽度
    CGFloat _lineChartWidth;
    // 底部报损成本的高度
    CGFloat _scrapLabelHeight;
    // 最高销量的坐标
    CGPoint _maxSalePoint;
    // 最低利润的坐标
    CGPoint _minProfitPoint;
}
// 头部利润、图例容器视图
@property (nonatomic, strong) UIView *headerView;
// 月度利润合计数量
@property (nonatomic, strong) UILabel *profitLabel;
// 报损合计
@property (nonatomic, strong) UILabel *scrapLabel;

/* 折线图区域 */
// 折线图容器
@property (nonatomic, strong) UIView *chartView;
// 坐标容器
@property (nonatomic, strong) UIView *axisView;
// X轴的y坐标
@property (nonatomic, assign) CGFloat xAxisYPoint;
// X轴线条
@property (nonatomic, strong) CAShapeLayer *xAxisLine;
// X轴端点
@property (nonatomic, strong) CAShapeLayer *xAxisSpot;
// Y轴线条
@property (nonatomic, strong) CAShapeLayer *yAxisLine;
// y轴顶部端点
@property (nonatomic, strong) CAShapeLayer *yAxisTopSpot;
// y轴底部端点
@property (nonatomic, strong) CAShapeLayer *yAxisBottomSpot;
// 日期label
@property (nonatomic, strong) NSMutableArray *dayLabels;
// 销售折线
@property (nonatomic, strong) CAShapeLayer *saleLine;
// 销售折线上的点，数组里的元素为CAShapeLayer
@property (nonatomic, strong) NSMutableArray *saleLineSpots;
// 利润折线
@property (nonatomic, strong) CAShapeLayer *profitLine;
// 利润折线上的点，数组里的元素为CAShapeLayer
@property (nonatomic, strong) NSMutableArray *profitLineSpots;
// 竖虚线，数组里的元素为CAShapeLayer
@property (nonatomic, strong) NSMutableArray *dottedLines;
// 最高销量
@property (nonatomic, strong) UILabel *maxSaleLabel;
// 最低利润
@property (nonatomic, strong) UILabel *minProfitLabel;

@end

@implementation LGLLineChartView

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = LGLChartBackgroundColor;
        _scrapLabelHeight = 37.5;
        _lineChartWidth = 42.0 * LGLChartScreenScale;
    }
    return self;
}

- (instancetype)initWithFrame:(CGRect)frame monthProfit:(CGFloat)monthProfit saleAmounts:(NSArray *)saleAmounts grossProfits:(NSArray *)grossProfits scrapCost:(CGFloat)scrapCost {
    if (self = [super initWithFrame:frame]) {
        self.backgroundColor = LGLChartBackgroundColor;
        _scrapLabelHeight = 37.5;
        _lineChartWidth = 42.0 * LGLChartScreenScale;
        
        self.monthProfit = monthProfit;
        self.saleAmounts = saleAmounts;
        self.grossProfits = grossProfits;
        self.scrapCost = scrapCost;
    }
    return self;
}

#pragma mark - 绘图方法
// 画折线图区域
- (void)drawLineChart {
    // 补零
    if (_saleAmounts.count > _grossProfits.count) {
        NSMutableArray *temp = [NSMutableArray arrayWithArray:_grossProfits];
        for (NSUInteger i = _grossProfits.count; i < _saleAmounts.count; i ++) {
            [temp addObject:@0];
        }
        _grossProfits = [temp copy];
    } else if (_saleAmounts.count < _grossProfits.count) {
        NSMutableArray *temp = [NSMutableArray arrayWithArray:_saleAmounts];
        for (NSUInteger i = _saleAmounts.count; i < _grossProfits.count; i ++) {
            [temp addObject:@0];
        }
        _saleAmounts = [temp copy];
    } else {
        
    }
 
    [self drawAxis];
    [self drawDaysLabel];
    [self drawSaleChart];
    [self drawProfitChart];
    [self drawDottedLines];
    [self drawExtremeValue];
}

// 头部利润、图例区域
- (UIView *)headerView {
    if (!_headerView) {
        _headerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.width, headerHeight)];
        _headerView.backgroundColor = [UIColor whiteColor];
        [self addSubview:_headerView];
        
        /* 利润 */
        // 标题
        UIButton *profixTitle = [[UIButton alloc] initWithFrame:CGRectMake(10, 10, 135, 20)];
        profixTitle.titleLabel.font = [UIFont systemFontOfSize:15.0];
        [profixTitle setTitle:@"月度毛利润合计" forState:UIControlStateNormal];
        [profixTitle setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        [profixTitle setImage:[UIImage imageNamed:@"money"] forState:UIControlStateNormal];
        profixTitle.titleEdgeInsets = UIEdgeInsetsMake(0, 5, 0, -5);
        [_headerView addSubview:profixTitle];
        // 利润(见- (UILabel *)profitLabel)
        
        /* 图例 */
        // 每日销售实收
        NSArray *colors = @[LGLChartBlueColor, LGLChartOrangeColor];
        NSArray *textes = @[@"每日毛利润", @"每日销售实收"];
        for (NSInteger i = 0; i < 2; i ++) {
            // 圆点
            UIView *spot = [[UIView alloc] initWithFrame:CGRectMake((190 + 90 * i) * LGLChartScreenScale, 37, 10, 10)];
            spot.backgroundColor = colors[i];
            spot.layer.cornerRadius = spot.width/2;
            spot.layer.masksToBounds = YES;
            [_headerView addSubview:spot];
            // 文本
            UILabel *content = [[UILabel alloc] initWithFrame:CGRectMake((205 + 90 * i) * LGLChartScreenScale, 35.5, 76 * LGLChartScreenScale, 13 * LGLChartScreenScale)];
            content.font = [UIFont systemFontOfSize:12.0 * LGLChartScreenScale];
            content.text = textes[i];
            content.textColor = LGLChartColor(83, 83, 83, 1);
            [_headerView addSubview:content];
        }
    }
    return _headerView;
}

// 利润数量
- (UILabel *)profitLabel {
    if (!_profitLabel) {
        _profitLabel = [[UILabel alloc] initWithFrame:CGRectMake(150, 10, self.headerView.width - 158, 20)];
        _profitLabel.font = [UIFont systemFontOfSize:17.0];
        _profitLabel.textColor = LGLChartYellowColor;
        _profitLabel.textAlignment = NSTextAlignmentRight;
        [self.headerView addSubview:_profitLabel];
    }
    return _profitLabel;
}

// 月度报损支出区域
- (UILabel *)scrapLabel {
    if (!_scrapLabel) {
        _scrapLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, self.height - _scrapLabelHeight - separatorLineHeight, self.width, _scrapLabelHeight)];
        _scrapLabel.backgroundColor = [UIColor whiteColor];
        _scrapLabel.textColor = [UIColor blackColor];
        _scrapLabel.font = [UIFont systemFontOfSize:15.0];
        [self addSubview:_scrapLabel];
    } else {
        _scrapLabel.frame = CGRectMake(0, self.height - _scrapLabelHeight - separatorLineHeight, self.width, _scrapLabelHeight);
    }
    return _scrapLabel;
}

// 折线图容器
- (UIView *)chartView {
    if (!_chartView) {
        _chartView = [[UIView alloc] initWithFrame:CGRectMake(0, self.headerView.height, self.width, self.height - self.headerView.height - self.scrapLabel.height - separatorLineHeight)];
        _chartView.backgroundColor = [UIColor whiteColor];
        _chartView.clipsToBounds = NO;
        [self addSubview:_chartView];
    }
    return _chartView;
}

// X轴、Y轴容器
- (UIView *)axisView {
    if (!_axisView) {
        _axisView = [[UIView alloc] initWithFrame:self.chartView.bounds];
        _axisView.backgroundColor = [UIColor clearColor];
        [_chartView addSubview:self.axisView];
        
        [self drawAxis];
    }
    return _axisView;
}

// 画X轴、Y轴
- (void)drawAxis {
    /* X轴 */
    // x轴上的线条
    NSArray *xAxisPoints = @[@[@(axisMargin), @(self.xAxisYPoint)], @[@(self.axisView.width - axisMargin), @(self.xAxisYPoint)]];
    [self.xAxisLine setPoints:xAxisPoints];
    
    // x轴的圆点
    CGPoint xSpotCenter = [LGLChartUtil pointFromArray:xAxisPoints[1]];
    UIBezierPath *xSpotPath = [UIBezierPath bezierPathWithArcCenter:xSpotCenter radius:axisSpotRadius startAngle:0 endAngle:M_PI * 2 clockwise:YES];;
    self.xAxisSpot.path = xSpotPath.CGPath;
    
    /* Y轴 */
    // y轴上的线条
    NSArray *yAxisPoints = @[@[@(axisMargin), @(axisSpotRadius)], @[@(axisMargin), @(self.axisView.height- axisSpotRadius)]];
    [self.yAxisLine setPoints:yAxisPoints];
    
    // y轴上端的圆点
    CGPoint yTopSpotCenter = [LGLChartUtil pointFromArray:yAxisPoints[0]];
    UIBezierPath *yTopSpotPath = [UIBezierPath bezierPathWithArcCenter:yTopSpotCenter radius:axisSpotRadius startAngle:0 endAngle:M_PI * 2 clockwise:YES];
    self.yAxisTopSpot.path = yTopSpotPath.CGPath;
    
    // y轴下端的圆点
    CGPoint yBottomSpotCenter = [LGLChartUtil pointFromArray:yAxisPoints[1]];
    UIBezierPath *yBottomSpotPath = [UIBezierPath bezierPathWithArcCenter:yBottomSpotCenter radius:axisSpotRadius startAngle:0 endAngle:M_PI * 2 clockwise:YES];
    self.yAxisBottomSpot.path = yBottomSpotPath.CGPath;
}

// x轴的y坐标
- (CGFloat)xAxisYPoint {
    if (!_saleAmounts.count && !_grossProfits.count) {
        _xAxisYPoint = self.axisView.height/2;
    } else {
        // 求最大值和最小值
        NSMutableArray *allData = [NSMutableArray arrayWithArray:_saleAmounts];
        [allData addObjectsFromArray:_grossProfits];
        CGFloat max = [LGLChartUtil maxNumber:allData];
        CGFloat min = [LGLChartUtil minNumber:allData];
        if (max < 0) {
            _xAxisYPoint = 0;
            // y轴上端的圆点清空
            self.yAxisTopSpot.fillColor = [UIColor clearColor].CGColor;
        } else if (min > 0){
            _xAxisYPoint = self.axisView.height - daysLabelHeight;
            /* 更新坐标 */
            // y轴上的线条
            NSArray *yAxisPoints = @[@[@(axisMargin), @(axisSpotRadius)], @[@(axisMargin), @(self.axisView.height- daysLabelHeight)]];
            [self.yAxisLine setPoints:yAxisPoints];
            // y轴下端的圆点清空
            self.yAxisBottomSpot.fillColor = [UIColor clearColor].CGColor;
        } else {
            _xAxisYPoint = max/(max + fabs(min)) * self.axisView.height - daysLabelHeight;
            if (max < fabs(min)) {
                _xAxisYPoint = (1 - fabs(min)/(max + fabs(min))) * self.axisView.height;
            }
            self.yAxisBottomSpot.fillColor = [UIColor blackColor].CGColor;
        }
    }
    return _xAxisYPoint;
}

// x轴的线条
- (CAShapeLayer *)xAxisLine {
    if (!_xAxisLine) {
        _xAxisLine = [CAShapeLayer drawLine:nil onLayer:self.axisView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:LGLChartLineWidth];
    }
    return _xAxisLine;
}

// x轴的端点
- (CAShapeLayer *)xAxisSpot {
    if (!_xAxisSpot) {
        
        _xAxisSpot = [CAShapeLayer layer];
        _xAxisSpot.fillColor = [UIColor blackColor].CGColor;
        CGPoint xSpotCenter = CGPointMake(self.axisView.width - axisMargin, self.xAxisYPoint);
        UIBezierPath *xSpotPath = [UIBezierPath bezierPathWithArcCenter:xSpotCenter radius:axisSpotRadius startAngle:0 endAngle:M_PI * 2 clockwise:YES];
        _xAxisSpot.path = xSpotPath.CGPath;
        [self.axisView.layer addSublayer:_xAxisSpot];
    }
    return _xAxisSpot;
}

// y轴的线条
- (CAShapeLayer *)yAxisLine {
    if (!_yAxisLine) {
        _yAxisLine = [CAShapeLayer drawLine:nil onLayer:self.axisView.layer strokeColor:[UIColor lightGrayColor].CGColor lineWidth:LGLChartLineWidth];
    }
    return _yAxisLine;
}

// y轴顶部端点
- (CAShapeLayer *)yAxisTopSpot {
    if (!_yAxisTopSpot) {
        _yAxisTopSpot = [CAShapeLayer layer];
        _yAxisTopSpot.fillColor = [UIColor blackColor].CGColor;
        CGPoint yTopSpotCenter = CGPointMake(axisMargin, axisSpotRadius);
        UIBezierPath *yTopSpotPath = [UIBezierPath bezierPathWithArcCenter:yTopSpotCenter radius:axisSpotRadius startAngle:0 endAngle:M_PI * 2 clockwise:YES];
        _yAxisTopSpot.path = yTopSpotPath.CGPath;
        [self.axisView.layer addSublayer:_yAxisTopSpot];
    }
    return _yAxisTopSpot;
}

// y轴底部端点
- (CAShapeLayer *)yAxisBottomSpot {
    if (!_yAxisBottomSpot) {
        _yAxisBottomSpot = [CAShapeLayer layer];
        _yAxisBottomSpot.fillColor = [UIColor blackColor].CGColor;
        CGPoint yBottomSpotCenter = CGPointMake(axisMargin, self.axisView.height - axisSpotRadius);
        UIBezierPath *yBottomSpotPath = [UIBezierPath bezierPathWithArcCenter:yBottomSpotCenter radius:axisSpotRadius startAngle:0 endAngle:M_PI * 2 clockwise:YES];
        _yAxisBottomSpot.path = yBottomSpotPath.CGPath;
        [self.axisView.layer addSublayer:_yAxisBottomSpot];
    }
    return _yAxisBottomSpot;
}

// 画X轴上的日期
- (NSMutableArray *)dayLabels {
    if (!_dayLabels) {
        _dayLabels = [NSMutableArray array];
        NSArray *days = @[@"1日", @"5日", @"9日", @"13日", @"17日", @"21日", @"25日", @"29日"];
        for (NSUInteger i = 0; i < days.count; i ++) {
            UILabel *dayLabel = [[UILabel alloc] initWithFrame:CGRectMake(_lineChartWidth * i, _xAxisYPoint, daysLabelWidth, daysLabelHeight)];
            dayLabel.font = [UIFont systemFontOfSize:12.0];
            dayLabel.text = days[i];
            dayLabel.textColor = [UIColor blackColor];
            dayLabel.textAlignment = NSTextAlignmentCenter;
            [self.axisView addSubview:dayLabel];
            
            [_dayLabels addObject:dayLabel];
        }
    }
    return _dayLabels;
}

-(void)drawDaysLabel{
    [self.dayLabels enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        UILabel *dayLabel = obj;
        dayLabel.frame = CGRectMake(dayLabel.x, self.xAxisYPoint, dayLabel.width, dayLabel.height);
    }];
}

// 每日销售的折线图
- (CAShapeLayer *)saleLine {
    if (!_saleLine) {
        _saleLine = [CAShapeLayer drawLine:nil onLayer:self.chartView.layer strokeColor:LGLChartOrangeColor.CGColor lineWidth:LGLChartLineWidth];
    }
    return _saleLine;
}

- (NSMutableArray *)saleLineSpots {
    if (!_saleLineSpots) {
        _saleLineSpots = [NSMutableArray array];
        for (NSUInteger i = 0; i < 31; i ++) {
            CAShapeLayer *spot = [CAShapeLayer layer];
            spot.fillColor = LGLChartOrangeColor.CGColor;
            [self.chartView.layer addSublayer:spot];
            [_saleLineSpots addObject:spot];
        }
    }
    return _saleLineSpots;
}

- (void)drawSaleChart {
    NSArray *allPoints = [self calculatePoints:_saleAmounts isSale:YES];
    [self.saleLine setPoints:allPoints];
    // 清除多余的点
    for (NSUInteger i = _saleAmounts.count; i < self.saleLineSpots.count; i ++) {
        CAShapeLayer *spot = self.saleLineSpots[i];
        spot.fillColor = [UIColor clearColor].CGColor;
    }
    // 画点
    [allPoints enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSArray *point = obj;
        CAShapeLayer *spot = self.saleLineSpots[idx];
        spot.fillColor = LGLChartOrangeColor.CGColor;
        UIBezierPath *path = [UIBezierPath bezierPathWithArcCenter:[LGLChartUtil pointFromArray:point] radius:lineSpotRadius startAngle:0 endAngle:M_PI * 2 clockwise:YES];
        spot.path = path.CGPath;
    }];
}

// 每日毛利润的折线图
- (CAShapeLayer *)profitLine {
    if (!_profitLine) {
        _profitLine = [CAShapeLayer drawLine:nil onLayer:self.chartView.layer strokeColor:LGLChartBlueColor.CGColor lineWidth:LGLChartLineWidth];
    }
    return _profitLine;
}

- (NSMutableArray *)profitLineSpots {
    if (!_profitLineSpots) {
        _profitLineSpots = [NSMutableArray array];
        for (NSUInteger i = 0; i < 31; i ++) {
            CAShapeLayer *spot = [CAShapeLayer layer];
            spot.fillColor = LGLChartBlueColor.CGColor;
            [self.chartView.layer addSublayer:spot];
            [_profitLineSpots addObject:spot];
        }
    }
    return _profitLineSpots;
}

- (void)drawProfitChart {
    NSArray *allPoints = [self calculatePoints:_grossProfits isSale:NO];
    [self.profitLine setPoints:allPoints];
    // 清除多余的点
    for (NSUInteger i = _grossProfits.count; i < self.profitLineSpots.count; i ++) {
        CAShapeLayer *spot = self.profitLineSpots[i];
        spot.fillColor = [UIColor clearColor].CGColor;
    }
    // 画点
    [allPoints enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        NSArray *point = obj;
        CAShapeLayer *spot = self.profitLineSpots[idx];
        spot.fillColor = LGLChartBlueColor.CGColor;
        UIBezierPath *path = [UIBezierPath bezierPathWithArcCenter:[LGLChartUtil pointFromArray:point] radius:lineSpotRadius startAngle:0 endAngle:M_PI * 2 clockwise:YES];
        spot.path = path.CGPath;
    }];
}

// 画虚线
- (NSMutableArray *)dottedLines {
    if (!_dottedLines) {
        _dottedLines = [NSMutableArray array];
        for (NSUInteger i = 0; i < 7; i ++) {
            CAShapeLayer *dotted = [CAShapeLayer drawDotted:nil pathClose:NO onLayer:self.axisView.layer attributes:DTLAttrMake([UIColor grayColor].CGColor, [UIColor clearColor].CGColor, LGLChartLineWidth, 3, 2)];
            [_dottedLines addObject:dotted];
        }
    }
    return _dottedLines;
}

- (void)drawDottedLines {
    NSArray *salePoints = [self calculatePoints:_saleAmounts isSale:YES];
    NSArray *profitPoints = [self calculatePoints:_grossProfits isSale:NO];
    NSUInteger showCount = (_saleAmounts.count - 1)/4;
    // 清除多余的虚线
    for (NSUInteger i = showCount; i < self.dottedLines.count; i ++) {
        CAShapeLayer *dotted = self.dottedLines[i];
        dotted.strokeColor = [UIColor clearColor].CGColor;
    }
    // 画虚线
    for (NSUInteger i = 0; i < showCount; i ++) {
        /* 计算虚线的起止点 */
        CGPoint salePoint = [LGLChartUtil pointFromArray:salePoints[(i + 1) * 4]];
        CGPoint profitPoint = [LGLChartUtil pointFromArray:profitPoints[(i + 1) * 4]];
        // 找出两点的最小y坐标（最大销售）
        CGFloat topPointY = salePoint.y < profitPoint.y ? salePoint.y : profitPoint.y;
        topPointY = topPointY > self.xAxisYPoint ? self.xAxisYPoint : topPointY;
        // 找出两点的最大y坐标（最小利润）
        CGFloat bottomPointY = profitPoint.y < salePoint.y ? salePoint.y : profitPoint.y;
        bottomPointY = bottomPointY < self.xAxisYPoint ? self.xAxisYPoint : bottomPointY;
        
        NSArray *points = @[@[@(salePoint.x), @(topPointY)], @[@(salePoint.x), @(bottomPointY)]];
        CAShapeLayer *dotted = self.dottedLines[i];
        [dotted setPoints:points];
        dotted.strokeColor = [UIColor grayColor].CGColor;
    }
}

// 画最大最小值
- (void)drawExtremeValue {
    
    // 显示最高销售或最低利润
    NSArray *salePoints = [self calculatePoints:_saleAmounts isSale:YES];
    CGFloat maxSale = [LGLChartUtil maxNumber:_saleAmounts];
    for (NSUInteger i = 0; i < _saleAmounts.count; i ++) {
        NSNumber *num = _saleAmounts[i];
        if ([num doubleValue] == maxSale) {
            _maxSalePoint = [LGLChartUtil pointFromArray:salePoints[i]];
        }
    }
    self.maxSaleLabel.text = [LGLChartUtil stringFromNumber:@(maxSale)];
    [_maxSaleLabel sizeToFit];
    CGFloat profitWidth = _maxSaleLabel.width + maxOrMinAmountLabelHeight;
    _maxSaleLabel.frame = CGRectMake(_maxSalePoint.x - profitWidth/2, _maxSalePoint.y - 20 - lineSpotRadius, profitWidth, 14);
    
    
    // 显示最低利润
    NSArray *profitPoints = [self calculatePoints:_grossProfits isSale:NO];
    CGFloat minProfit = [LGLChartUtil minNumber:_grossProfits];
    for (NSUInteger i = 0; i < _saleAmounts.count; i ++) {
        NSNumber *num = _grossProfits[i];
        if ([num doubleValue] == minProfit) {
            _minProfitPoint = [LGLChartUtil pointFromArray:profitPoints[i]];
        }
    }
    self.minProfitLabel.text = [LGLChartUtil stringFromNumber:@(minProfit)];
    [_minProfitLabel sizeToFit];
    CGFloat saleWidth = _minProfitLabel.width + maxOrMinAmountLabelHeight;
    _minProfitLabel.frame = CGRectMake(_minProfitPoint.x - saleWidth/2, _minProfitPoint.y + 2 + lineSpotRadius, saleWidth, 14);
}

// 最大销售额
- (UILabel *)maxSaleLabel {
    if (!_maxSaleLabel) {
        _maxSaleLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _maxSaleLabel.backgroundColor = LGLChartOrangeColor;
        _maxSaleLabel.font = [UIFont systemFontOfSize:12];
        _maxSaleLabel.text = [LGLChartUtil stringFromNumber:@([LGLChartUtil maxNumber:_saleAmounts])];
        _maxSaleLabel.textColor = [UIColor whiteColor];
        _maxSaleLabel.textAlignment = NSTextAlignmentCenter;
        [self.chartView addSubview:_maxSaleLabel];
    }
    return _maxSaleLabel;
}

// 最少利润
- (UILabel *)minProfitLabel {
    if (!_minProfitLabel) {
        _minProfitLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _minProfitLabel.backgroundColor = LGLChartBlueColor;
        _minProfitLabel.font = [UIFont systemFontOfSize:12];
        _minProfitLabel.text = [LGLChartUtil stringFromNumber:@([LGLChartUtil minNumber:_grossProfits])];
        _minProfitLabel.textColor = [UIColor whiteColor];
        _minProfitLabel.textAlignment = NSTextAlignmentCenter;
        [self.chartView addSubview:_minProfitLabel];
    }
    return _minProfitLabel;
}

#pragma mark - Setter

- (void)setMonthProfit:(CGFloat)monthProfit {
    _monthProfit = monthProfit;
    self.profitLabel.text = [LGLChartUtil stringFromNumber:@(_monthProfit)];
}

- (void)setSaleAmounts:(NSArray *)saleAmounts {
    if ([saleAmounts isEqualToArray:_saleAmounts]) {
        return;
    }
    _saleAmounts = saleAmounts;
}

- (void)setGrossProfits:(NSArray *)grossProfits {
    if ([grossProfits isEqualToArray:_grossProfits]) {
        return;
    }
    _grossProfits = grossProfits;
}

- (void)setScrapCost:(CGFloat)scrapCost {
    _scrapCost = scrapCost;
    if (scrapCost != 0) {
        _scrapLabelHeight = 37.5;
    } else {
        _scrapLabelHeight = 0;
    }
    self.scrapLabel.text = [NSString stringWithFormat:@"    月度报损成本支出合计   %@", [LGLChartUtil stringFromNumber:@(-_scrapCost)]];
    [self drawLineChart];
}

#pragma mark - 工具方法

// 计算出数组中数据对应的所有的点，isSale：标记变量，是销售额还是毛利润
- (NSArray *)calculatePoints:(NSArray *)datas isSale:(BOOL)isSale {
    NSMutableArray *points = [NSMutableArray array];
    // 最大高度，所有高度以此为基准
    CGFloat maxHeight = self.xAxisYPoint * 0.85;
    //    if (!isSale) {
    //        // 是计算毛利润的店
    //        maxHeight *= 0.6;
    //    }
    // 最大数值，任意数值num的y值：num／maxNum * maxHeight
    CGFloat maxNum = MAX(LGLChartZero, fabs([LGLChartUtil maxAbsolute:datas]));
    for (NSUInteger i = 0; i < datas.count; i ++) {
        CGFloat x = axisMargin + i * _lineChartWidth/4;
        CGFloat value = [datas[i] floatValue];
        CGFloat y = self.xAxisYPoint - value / maxNum * maxHeight;
        [points addObject:@[@(x), @(y)]];
    }
    return [points copy];
}


/*
 // Only override drawRect: if you perform custom drawing.
 // An empty implementation adversely affects performance during animation.
 - (void)drawRect:(CGRect)rect {
 // Drawing code
 }
 */

@end
